function A = create_try
%Create Sudoku enigma of high difficulty by random algorithm.
%The created Sudoku is solvable with help of sudoku_try_func.m but not solvable solely by sudoku_excl_func.
%Input: nothing, Output: matrix A of initial values for Sudoku.

% Copyright (C) 2007 Felix Voigt.
% For full notice see file "sudoku_func.m".

sudoku_found = 0; %Flag, signalizing whether Sudoku has been found,
%which can be uniquely solved and not solely be solved by excl. principles.
rand('state',sum(100*clock))  %Sets seed of random numbers related to computer time.
while sudoku_found == 0
    A = zeros(9,9);
    A_old = zeros(9,9); %Old values are stored, to provide the possibility for a back step.
    %(For the case, that the new initial values effect an unsolvable Sudoku.)
    B = zeros(9,9,9);
    B_old = zeros(9,9,9);
    
    err = 0; %Flag, signalizing an error in initial conditions of Sudoku.
    solved_excl = 0; %Flag, signalizing whether Sudoku can be solved solely by exclusion principles.
    solved_try = 0; %Flag, signalizing whether Sudoku can be solved by sudoku_try_func.m.
    guesses = 0;%Number of guesses performed by sudoku_try_func.m.
    s = 0; %# of solutions to Sudoku.
    create_end = 0; %Flag, set when sudoku_found = 1, or when occB = 729.

    while create_end == 0
        err = 0; %Initialization.
        solved_excl = 0;
        solved_try = 0;
        guesses = 0;
        s = 0;
        x = ceil(9*rand); %Random integer between 1 and 9.
        y = ceil(9*rand); %"
        z = ceil(9*rand); %"
        
        if B(x,y,z) == 0 %Check, whether digit z is possible at point (x,y) of matrix A.
            A_old = A; %Store old Matrizes.
            B_old = B;
            A(x,y) = z;
            [B_excl,C_excl,D_excl,solved_excl,err] = sudoku_excl_func(A);
            B = B_old | B_excl; %Logical OR operation in order to keep excluded random choices.
            if err == 1
                A = A_old;
                B = B_old;
                B(x,y,z) = 1; %Exclude last random choice.
            else
                if solved_excl == 1 %If Sudoku can be solved solely by exclusion principles, then step back.
                    A = A_old;
                    B = B_old;
                    B(x,y,z) = 1; %Exclude last random choice.
                else
                    [B_try,C_try,D_try,guesses,solved_try,err] = sudoku_try_func(A,B_excl,C_excl);
                    if solved_try == 1
                        %Up to now it is not clear, whether Sudoku is uniquely solvable.
                        S =  sudoku_rec_func(D_excl); %Recursive solution based on presolution by exclusion principle.
                        s = size(S,3);
                        if s == 1 %One unique solution.                    
                            sudoku_found = 1
                            create_end = 1
                            A
                        else %If there are multiple solutions, then step back.
                            %Otherwise all Sudokus would be solved by sudoku_excl_func.m in the following.
                            A = A_old;
                            B = B_old;
                            B(x,y,z) = 1; %Exclude last random choice.
                        end %if
                    end %if
                end %if
            end %if
            occA = sum(sum(A&ones(9,9))); %Number of places in A, occupied by a nonzero digit.
            occB = sum(sum(sum(B&ones(9,9,9)))); %Number of places in matrix B, occupied by a nonzero number.
            occA_occB_err_solvedexcl_solvedtry_guesses_s = [occA,occB,err,solved_excl,solved_try,guesses,s]
        end %if
        if occB == 729
            create_end = 1
        end %if
    end %while
    
end %while

